/*
  Tvister - Live status checker
  Copyright(c) 2011 Ndragomirov[jdavid214@gmail.com]
  MIT Licensed
  
  Dual licensed under the MIT and GPL licenses:
  http://www.opensource.org/licenses/mit-license.php
  http://www.gnu.org/licenses/gpl.html
  
*/

var HTTP        = require('http'),
	URL         = require('url'),
	querystring = require('querystring');


function Agent() {
	this._timeout = 1;
};

function shouldDropConn(cb) {
//	if ( this._request ) {
//		try { 
//			//this._request.abort(); 
//			//res.removeAllListeners(['data', 'end', 'close']);
//			cb && cb( null, { error: 'response error' });
//		}
//		catch (ex) {}
//		
//		console.log('drop connection');
//	};
};

Agent.types = {
	xml: 'text/xml',
	json: 'application/json'
};

Agent.parser = {
	'application/json': JSON.parse,
	'text/xml': function() {}
};

Agent.prototype.setOptions = function( url ) {
	var parsedUrl = URL.parse( url );
	console.log('url: ', url,  'hostname: ', parsedUrl.hostname, 
				'pathname: ', parsedUrl.pathname + ( this._query || '' ), 'port: ', parsedUrl.port || 80);
	
	return {
		headers: {
			'Accept': 'application/json', 
			'X-Request': 'JSON'
		},
		host: parsedUrl.hostname,
		path: parsedUrl.pathname + ( this._query || '' ),
		port: parsedUrl.port || 80
	};
	
};

function isObject(obj) {
    var cons = obj.constructor;
    return cons && 'Object' == cons.name;
};

Agent.prototype.options = function(options) {
	this._query = isObject(options) ? 
					'?' + querystring.stringify(options) : 
					'';
	return this;
};

Agent.prototype.setHeaders = function(res) {
	var contentType = ( res.headers['content-type'] || '' ).split(/ *; */);
	this.contentType = contentType;
};

Agent.prototype.setStatus = function(res) {
	var status = res.statusCode;
	this.error = status == 4 || status == 5;
};

Agent.prototype.parseBody = function(data) {
	var parseFn = Agent.parser[this.contentType];
	return parseFn(data);
};

Agent.prototype.timeout = function(value) {
	this._timeout = value;
	return this;
};

function onClose(res) {
	res.removeAllListeners(['data', 'end', 'close']);
};

Agent.prototype.onResponse = function( res ) {
	
	var data = '',
		self = this;
	
	//res.setEncoding('utf8');
	this.setHeaders(res);
	this.setStatus(res);
	
	res.on('data', function(chunk) {
		data += chunk;
	});
	
//	res.on('close', function() {
//		
//	});
	
	res.on('end', function() {
		res.removeAllListeners(['data', 'end', 'close']);
		//console.log('DATA: ', data);
		data = self.error ? { error: 'response error' } : data;
		self.callback && self.callback( null, data);
	});
	
};

Agent.prototype.send = function( url, cb ) {
	
	var self = this,
		tid = setTimeout( shouldDropConn.bind(self, cb), self._timeout * 1000 );
	
	this.callback = cb;
	
	this._request = HTTP.get( self.setOptions( url ), function(res) {
		clearTimeout(tid);
		self.onResponse(res);	
	});
	
};


module.exports = Agent;