/*
  Tvister - Live status checker
  Copyright(c) 2011 Ndragomirov[jdavid214@gmail.com]
 
  Dual licensed under the MIT and GPL licenses:
  http://www.opensource.org/licenses/mit-license.php
  http://www.gnu.org/licenses/gpl.html
  
 */

var xml2js = require('xml2js'), embedy = require('./embedy');

var parseError = {
	error : 'parse error'
};

var apis = {};

apis.justin = {
	api : 'http://api.justin.tv/api/stream/list.json',

	channelTag : 'channel',

	embed : function(id) {
		var objectOptions = {
			attr : {
				data : 'http://www.justin.tv/widgets/live_embed_player.swf?channel={channelid}'
						.replace(/{channelid}/, id)
			},
			param : {
				movie : "http://www.justin.tv/widgets/live_embed_player.swf",
				flashvars : 'hostname=www.justin.tv&channel={channelid}&auto_play=false&start_volume=25'
						.replace(/{channelid}/, id)
			}
		};

		return embedy.create(objectOptions);
	},

	parse : function(data, cb) {
		var self = this;

		console.log('parsing justin response', data);

		try {
			data = JSON.parse(data);
		} catch (e) {
			cb && cb(null, parseError);
		}

		data = data.map(function(ch) {
			var id = ch.channel.login;
			return {
				thumbnail : ch.channel.screen_cap_url_medium,
				id : id,
				embed : self.embed(id),
				viewers : ch.channel_count,
				link : ch.channel.channel_url,
			};
		});

		cb && cb(null, data);
	}
};

apis.own3d = {
	api : 'http://api.own3d.tv/live.php',

	embed : function(id) {
		var objectOptions = {
			param : {
				movie : 'http://www.own3d.tv/livestream/{channelid}'.replace(
						/{channelid}/, id)
			}
		};

		var embedOptions = {
			attr : {
				src : objectOptions.param.movie
			}
		};
		return embedy.create(objectOptions, embedOptions);
	},

	channelTag : 'whatever',

	parse : function(data, cb) {

		var parser = new xml2js.Parser(), self = this;

		parser.on('error', function() {
			cb & cb(null, parseError);
		});

		parser.on('end', function(data) {
			data = data.channel.item;

			data = data.map(function(ch) {
				var id = ch.link.replace(/.*\/live\/([0-9]+?)/, '$1');
				return {
					thumbnail : ch.thumbnail,
					embed : self.embed(id),
					id : id,
					viewers : ch.misc['@'].viewers,
					link : ch.link,
				};
			});

			cb && cb(null, data);
		});

		parser.parseString(data);

	}
};

apis.regame = {
	api : 'http://www.regame.tv/liveview_xml_multiple.php',

	channelTag : 'stream_ids',

	// TODO regame embed support
	embed : function() {
	},

	parse : function(data, cb) {

		var parser = new xml2js.Parser(), self = this;

		parser.on('error', function() {
			cb & cb(null, parseError);
		});

		parser.on('end', function(data) {

			data = data.stream;

			if (!Array.isArray(data)) {
				data = [ data ];
			}
			;

			data = data.filter(function(ch) {
				return ch.online === 'true';
			}).map(function(ch) {
				var id = ch['@'].id;
				return {
					embed : self.embed(),
					thumbnail : ch.thumbnail,
					id : id,
					viewers : ch.viewer,
					link : ch.streamlink,
				};
			});

			cb && cb(null, data);
		});

		parser.parseString(data);
	}
};

apis.livestream = {
	api : 'http://channel.api.livestream.com/1.0/channelsinfo',

	thumb : 'http://thumbnail.api.livestream.com/thumbnail?name={channelid}',

	embed : function(id) {
		var objectOptions = {
			param : {
				movie : "http://cdn.livestream.com/grid/LSPlayer.swf?channel={channelid}&autoPlay=false"
						.replace(/{channelid}/, id)
			}
		};

		var embedOptions = {
			attr : {
				src : objectOptions.param.movie
			}
		};

		return embedy.create(objectOptions, embedOptions);
	},

	channelTag : 'channel',

	parse : function(data, cb) {

		var parser = new xml2js.Parser(), self = this;

		parser.on('error', function() {
			cb & cb(null, parseError);
		});

		parser.on('end', function(data) {

			data = data['ls:channel'];

			if (!Array.isArray(data)) {
				data = [ data ];
			};

			data = data.filter(function(ch) {
				return ch['ls:isLive'] === 'true';
			}).map(function(ch) {
				var id = ch['ls:shortName'];
				return {
					thumbnail : self.thumb.replace('{channelid}', id),
					embed : self.embed(id),
					id : id,
					viewers : ch['ls:currentViewerCount'],
					link : ch['ls:link'],
				};
			});
			cb && cb(null, data);
		});

		parser.parseString(data);
	}
};

apis.twitch = {
	api : apis.justin.api,
	channelTag : apis.justin.channelTag,
	parse : apis.justin.parse,

	embed : function(id) {
		var objectOptions = {
			attr : {
				data : 'http://www.twitch.tv/widgets/live_embed_player.swf?channel={channelid}'
						.replace(/{channelid}/, id)
			},
			param : {
				movie : "http://www.twitch.tv/widgets/live_embed_player.swf",
				flashvars : 'hostname=www.twitch.tv&channel={channelid}&auto_play=false&start_volume=25'
						.replace(/{channelid}/, id)
			}
		};

		return embedy.create(objectOptions);
	}

};

apis.xfire = {
	api : apis.livestream.api
};

module.exports = apis;